
#include "alt_types.h"
#include "opencores_i2c_regs.h"
#include "opencores_i2c.h"

//*************************************************************************************************/
/* @par Description: This function initializes the prescaler for the SCL and then enables the core.
 * @par Description: This must be run before any other i2c code is executed.
 *
 * @param base : the base address of the component
 * @param clk : freuqency of the clock driving this component  ( in Hz)
 * @param speed : SCL speed ie 100K, 400K ...            (in Hz)
 *
 * @return: None
 */
/**************************************************************************************************/
void I2C_init(alt_u32 base,alt_u32 clk,alt_u32 speed)
{
	  alt_u32 prescale = (clk/( 5 * speed))-1;
	  IOWR_OPENCORES_I2C_CTR(base, 0x00);							/* turn off the core*/
	  IOWR_OPENCORES_I2C_CR(base, OPENCORES_I2C_CR_IACK_MSK);		/* clearn any pening IRQ*/
	  IOWR_OPENCORES_I2C_PRERLO(base, (0xff & prescale));			/* load low presacle bit*/
	  IOWR_OPENCORES_I2C_PRERHI(base, (0xff & (prescale>>8)));		/* load upper prescale bit */
	  IOWR_OPENCORES_I2C_CTR(base, OPENCORES_I2C_CTR_EN_MSK);		/* turn on the core*/
}

//*************************************************************************************************/
/* @par Description: Sets the start bit and then sends the first byte which is
 * @par Description: the address of the device + the write bit.
 *
 * @param base : the base address of the component
 * @param add : address of I2C device
 * @param read :  1== read    0== write
 *
 * @return 0 : if address is acknowledged
 * @return 1 : if address was not acknowledged
 */
/**************************************************************************************************/
int I2C_start(alt_u32 base, alt_u32 add, alt_u32 read)
{
	IOWR_OPENCORES_I2C_TXR(base, ((add<<1) + (0x1 & read)));			/* transmit the address shifted by one and the read/write bit*/
	IOWR_OPENCORES_I2C_CR(base, OPENCORES_I2C_CR_STA_MSK |
								OPENCORES_I2C_CR_WR_MSK );				/* set start and write  bits which will start the transaction*/
	while( IORD_OPENCORES_I2C_SR(base) & OPENCORES_I2C_SR_TIP_MSK);		/* wait for the trnasaction to be over.*/
	if(IORD_OPENCORES_I2C_SR(base) & OPENCORES_I2C_SR_RXNACK_MSK)		/* now check to see if the address was acknowledged */
	{

		return (I2C_NOACK);
	}
	else
	{

		return (I2C_ACK);
	}
}

//*************************************************************************************************/
/* @par Description: Assumes that any addressing and start has already been done.
 * @par Description: Reads one byte of data from the slave.
 * @par Description: On the last read  we don't acknowledge and set the stop bit.
 *
 * @param base : the base address of the component
 * @param last : on the last read there must not be a ack
 *
 * @return : byte read back
 */
/**************************************************************************************************/
alt_u32 I2C_read(alt_u32 base,alt_u32 last)
{
	if( last)
	{
		IOWR_OPENCORES_I2C_CR(base, OPENCORES_I2C_CR_RD_MSK |
									OPENCORES_I2C_CR_NACK_MSK |
									OPENCORES_I2C_CR_STO_MSK);		/* start a read and no ack and stop bit*/
	}
	else
	{
		IOWR_OPENCORES_I2C_CR(base, OPENCORES_I2C_CR_RD_MSK );		/* start read*/
	}
	while( IORD_OPENCORES_I2C_SR(base) & OPENCORES_I2C_SR_TIP_MSK);	/* wait for the transaction to be over.*/

	return (IORD_OPENCORES_I2C_RXR(base));							/* now read the data */
}

//*************************************************************************************************/
/* @par Description: Assumes that any addressing and start has already been done.
 * @par Description: Writes one byte of data to the slave.
 * @par Description: If last is set the stop bit set.
 *
 * @param base : the base address of the component
 * @param data : byte to write
 * @param last : on the last write there must not be a ack
 *
 * @return : byte read back.
 */
/**************************************************************************************************/
alt_u32 I2C_write(alt_u32 base,alt_u8 data, alt_u32 last)
{
	IOWR_OPENCORES_I2C_TXR(base, data);/* transmit the data*/
	if( last)
	{
		IOWR_OPENCORES_I2C_CR(base, OPENCORES_I2C_CR_WR_MSK |
									OPENCORES_I2C_CR_STO_MSK);		/* start a read and no ack and stop bit*/
	}
	else
	{
		IOWR_OPENCORES_I2C_CR(base, OPENCORES_I2C_CR_WR_MSK );		/* start read*/
	}
	while( IORD_OPENCORES_I2C_SR(base) & OPENCORES_I2C_SR_TIP_MSK);	/* wait for the transaction to be over.*/
	if(IORD_OPENCORES_I2C_SR(base) & OPENCORES_I2C_SR_RXNACK_MSK)	/* now check to see if the address was acknowledged */
	{

		return (I2C_NOACK);
	}
	else
	{

		return (I2C_ACK);
	}
}
